import os

SYNCGIT_CMD_TIMEOUT = int(os.getenv("SYNCGIT_CMD_TIMEOUT", "15"))
SYNCGIT_REPO_DIR_NAME = os.getenv("SYNCGIT_REPO_DIR_NAME", ".repos")
SYNCGIT_DEFAULT_POLL_INTERVAL = int(os.getenv("SYNCGIT_POLL_INTERVAL", "5"))
